/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#include <aalib.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <stdio.h>
#include "aa_helper.h"
#include "aa_helperint.h"

struct aa_outputfile *aa_openoutputfile(aa_context *context, const char *filename) {
	struct aa_outputfile *f;
	f = (struct aa_outputfile *)malloc(sizeof(struct aa_outputfile));

	if(context == NULL || filename == NULL) {
		return NULL;
	}
	f->file = fopen(filename, "wb");
	if(f->file == NULL) {
		free(f);
		return NULL;
	}

	fprintf(f->file, "Width: %i\n"
						"Height: %i\n"
						"NormalAttr: %i\n"
						"DimAttr: %i\n"
						"BoldAttr: %i\n"
						"BoldFontAttr: %i\n"
						"ReverseAttr: %i\n"
						"SpecialAttr: %i\n"
						"EndHeader\n",
			aa_scrwidth(context), aa_scrheight(context),
			AA_NORMAL, AA_DIM, AA_BOLD, AA_BOLDFONT, AA_REVERSE, AA_SPECIAL);

	gettimeofday(&(f->start),NULL);
	return f;
}

int aa_writeframe(aa_context *context, struct aa_outputfile *outputfile) {
	int x,y;
	char formatstring[10];
	struct timeval now;
	int secs, usecs;

	__AA_CONST unsigned char *attrs;
	__AA_CONST unsigned char *text;

	if(outputfile == NULL || context == NULL) {
		return 1;
	}

	gettimeofday(&now,NULL);

	memset(formatstring, '\0', sizeof(formatstring));
	sprintf(formatstring, "%%%is\n", aa_scrwidth(context));

	text = aa_text(context);
	attrs = aa_attrs(context);

	secs = now.tv_sec - outputfile->start.tv_sec;
	usecs = now.tv_usec - outputfile->start.tv_usec;

	while(usecs < 0) {
		secs -= 1;
		usecs += 1000000;
	}

	fprintf(outputfile->file, "Time: %i.%i\n", secs, usecs);

	fprintf(outputfile->file, "Text:\n");
	for(y=0; y<aa_scrheight(context); y++) {
		for(x=0;x<aa_scrwidth(context);x++) {
			fprintf(outputfile->file, "%c",
				*(text + y * aa_scrwidth(context) + x));
		}
		fprintf(outputfile->file, "\n");
	}

	fprintf(outputfile->file, "Attributes:\n");
	for(y=0; y<aa_scrheight(context); y++) {
		for(x=0;x<aa_scrwidth(context);x++) {
			fprintf(outputfile->file, "%i",
				(int)*(attrs + y * aa_scrwidth(context) + x));
		}
		fprintf(outputfile->file, "\n");
	}
	fprintf(outputfile->file, "EndFrame\n");
	fprintf(outputfile->file, "\n\n");
	return 0;
}

void aa_closeoutputfile(struct aa_outputfile *outputfile) {
	if(outputfile != NULL) {
		fclose(outputfile->file);
		free(outputfile);
		outputfile = NULL;
	}
}


struct aa_inputfile *aa_openinputfile(const char *filename) {

	struct aa_inputfile *f;
	char line[200];

	f = (struct aa_inputfile *)malloc(sizeof(struct aa_inputfile));

	if(filename == NULL) {
		return NULL;
	}
	f->file = fopen(filename, "rb");
	if(f->file == NULL) {
		free(f);
		return NULL;
	}

	f->normal = AA_NORMAL;
	f->dim = AA_DIM;
	f->bold = AA_BOLD;
	f->boldfont = AA_BOLDFONT;
	f->reverse = AA_REVERSE;
	f->special = AA_SPECIAL;
	f->width = 80;
	f->height = 25;

	while(!feof(f->file)) {
		if(fgets(line, sizeof(line), f->file) == NULL) continue;
		if(strstr(line, "EndHeader") != NULL) break;
		if(1 == sscanf(line, "Width: %i", &(f->width))) {
			printf("Width: %i\n", (f->width));
		} else if(1 == sscanf(line, "Height: %i", &(f->height))) {
			printf("Height: %i\n", (f->height));
		} else if(1 == sscanf(line, "NormalAttr: %i", &(f->normal))) {
			printf("NormalAttr: %i\n", (f->normal));
		} else if(1 == sscanf(line, "DimAttr: %i", &(f->dim))) {
			printf("DimAttr: %i\n", (f->dim));
		} else if(1 == sscanf(line, "BoldAttr: %i", &(f->bold))) {
			printf("BoldAttr: %i\n", (f->bold));
		} else if(1 == sscanf(line, "BoldFontAttr: %i", &(f->boldfont))) {
			printf("BoldFontAttr: %i\n", (f->boldfont));
		} else if(1 == sscanf(line, "ReverseAttr: %i", &(f->reverse))) {
			printf("ReverseAttr: %i\n", (f->reverse));
		} else if(1 == sscanf(line, "SpecialAttr: %i", &(f->special))) {
			printf("SpecialAttr: %i\n", (f->special));
		}
	}


	return f;
}

int aa_readframe(struct aa_inputfile *inputfile,
	unsigned char *textbuf, unsigned char *attrbuf) {

	int in_text_buffer = 0;
	int in_attr_buffer = 0;
	int curr_text_buflen = 0;
	int curr_attr_buflen = 0;
	int i;
	char line[200];
	int bufsize;

	bufsize = inputfile->width * inputfile->height;

	memset((void *)attrbuf,'\0',bufsize);
	memset((void *)textbuf,'\0',bufsize);

	while(!feof(inputfile->file)) {
		if(fgets(line, sizeof(line), inputfile->file) == NULL) continue;
		if(strstr(line, "Text:") == line) {
			/* Found a new text buffer */
			in_text_buffer = 1;
			in_attr_buffer = 0;
		} else if(strstr(line, "Attributes:") == line) {
			/* Found a new attributes buffer */
			in_text_buffer = 0;
			in_attr_buffer = 1;
		} else if(strstr(line, "EndFrame") == line) {
			return 0;
		} else {
			char *endline;
			if(NULL != (endline = strstr(line, "\n"))) {
				*endline = '\0';
			}
			if(in_text_buffer && (curr_text_buflen + strlen(line)) <= bufsize) {
				strcat((char *)textbuf, line);
				curr_text_buflen += strlen(line);
			} else if (in_attr_buffer && (curr_attr_buflen + strlen(line)) <= bufsize) {
				for(i=0;i<strlen(line);i++) {
					if(line[i] == inputfile->normal) {
						attrbuf[i+curr_attr_buflen] = AA_NORMAL;
					} else if(line[i] == inputfile->dim) {
						attrbuf[i+curr_attr_buflen] = AA_DIM;
					} else if(line[i] == inputfile->bold) {
						attrbuf[i+curr_attr_buflen] = AA_BOLD;
					} else if(line[i] == inputfile->boldfont) {
						attrbuf[i+curr_attr_buflen] = AA_BOLDFONT;
					} else if(line[i] == inputfile->reverse) {
						attrbuf[i+curr_attr_buflen] = AA_REVERSE;
					} else if(line[i] == inputfile->special) {
						attrbuf[i+curr_attr_buflen] = AA_SPECIAL;
					} else {
						attrbuf[i+curr_attr_buflen] = AA_NORMAL;
					}
				}
				curr_attr_buflen += strlen(line);
			}
		}
	}
	/* File ended but we didn't return via EndFrame - frame may or
	may not be complete. */
	return 1;
}

void aa_closeinputfile(struct aa_inputfile *inputfile) {
	if(inputfile != NULL) {
		fclose(inputfile->file);
		free(inputfile);
		inputfile = NULL;
	}
}
